﻿;/////////////////////////////////////////////////////////////////////////////////
;***COMate***  COM automation through iDispatch.
;*===========
;*
;*Excel - fill sheet from a 2-d Safearray demo.
;*
;*Using a Safearray allows you to populate a range of cells with a single method call.
;*In this demo we use a safearray of signed integers, but you could use a safearray of variants
;*in order to populate cells with different types of data etc.
;/////////////////////////////////////////////////////////////////////////////////

IncludePath "..\"
XIncludeFile "COMatePLUS.pbi"


;-Safe array structures - taken from "VariantHelper_Include.pb"
;==============================================================
CompilerIf Defined(SAFEARRAYBOUND, #PB_Structure)=0
  Structure SAFEARRAYBOUND 
    cElements.l 
    lLbound.l 
  EndStructure 
CompilerEndIf

CompilerIf Defined(SAFEARRAY, #PB_Structure)=0
  Structure SAFEARRAY 
    cDims.w 
    fFeatures.w 
    cbElements.l 
    cLocks.l 
    *pvData.pData 
    rgsabound.SAFEARRAYBOUND[0] 
  EndStructure 
CompilerEndIf


;-Set up a 2-d safearray of signed integers of dimensions 20 by 20.
;==================================================================
  ;Begin with an array of SAFEARRAYBOUND structures; one for each dimension.
    Dim safeArrayBound.SAFEARRAYBOUND(1)
      With safeArrayBound(0)
        \lLbound = 1
        \cElements = 20
      EndWith
      With safeArrayBound(1)
        \lLbound = 1
        \cElements = 20
      EndWith
  ;Now create the array and check for success.
    *safeArray.SAFEARRAY = SafeArrayCreate_(#VT_I4, 2, @safeArrayBound())
    If *safeArray = 0
      MessageRequester("COMate -Excel 2-d Safearray demo!", "Couldn't create the Safearray!")
      End
    EndIf
  ;Populate the array.
    Dim indices.l(1)
    For i = 1 To 20
      For j = 1 To 20
        indices(0) = i
        indices(1) = j
        temp = 20*(i-1)+j
        SafeArrayPutElement_(*safeArray, @indices(), @temp)
      Next
    Next
  ;Bundle the Safearray up into a variant suitable for passing to a COM method.
    Define var.VARIANT
    With var
      \vt = #VT_ARRAY|#VT_I4
      \parray = *safeArray
    EndWith


;-Fire up Excel and load cells from the array in one go.
;=======================================================

Define.COMateObject ExcelObject, WorkBook
ExcelObject = COMate_CreateObject("Excel.Application")

If ExcelObject
  If ExcelObject\SetProperty("Visible = #True") = #S_OK
    WorkBook = ExcelObject\GetObjectProperty("Workbooks\Add")
    If WorkBook
        ExcelObject\SetProperty("Range('A1:T20') = " + Str(var) + " as variant")
      ExcelObject\Invoke("Quit()") 
      WorkBook\Release()
    EndIf
  EndIf
  ExcelObject\Release()
Else
  MessageRequester("COMate -Excel 2-d Safearray demo!", "Couldn't create the application object!")
EndIf

;Now free the Safearray. Either use SafeArrayDestroy_() or VariantClear_() on any variant containing the array.
  VariantClear_(var)

; IDE Options = PureBasic 5.21 LTS (Windows - x64)
; ExecutableFormat = Shared Dll
; CursorPosition = 55
; FirstLine = 47
; Folding = -
; EnableUnicode
; Executable = nxReportU.dll