﻿;/////////////////////////////////////////////////////////////////////////////////
;***COMate***  COM automation through iDispatch.
;*===========
;*
;*Excel demo.
;*
;*This demo shows one way of using a pre-compiled statement to write data to an array of cells within an Excel spreadsheet.
;*Using a statement in this case does not speed up proceedings because Excel caches all writes anyhow, but the demo does show how
;*to use 'bound parameters' within a statement via the BYREF modifier.
;*This means we do not need to modify the statement parameters ourselves, we just need to modify the variables which the parameters are bound
;*to instead!
;/////////////////////////////////////////////////////////////////////////////////

IncludePath "..\..\"
XIncludeFile "COMatePLUS.pbi"

Define.COMateObject ExcelObject, WorkBook


;Prepare our statement for writing to an individual cell. Our choice of cell is unimportant.
;What is important is that we use type-modifiers to set the parameter types.
;===========================================================================================
  hStatement = COMate_PrepareStatement("Cells(" + Str(@row) + " As long BYREF," + Str(@col) + " As long BYREF) = 1 As long")
  If hStatement = 0
    MessageRequester("COMatePLUS!", "Couldn't create the statement!" + #CRLF$ + #CRLF$ + "COMatePLUS error : " + COMate_GetLastErrorDescription())
    End
  EndIf


ExcelObject = COMate_CreateObject("Excel.Application")
If ExcelObject
  If ExcelObject\SetProperty("Visible = #True") = #S_OK
    WorkBook = ExcelObject\GetObjectProperty("Workbooks\Add")
    If WorkBook
      ;Write 10000 cells using our pre-compiled statement.
      ;We fill each of the 10000 cells with the value 1.
        For row = 1 To 100
          For col = 1 To 100
            ExcelObject\SetProperty("", hStatement)
          Next
        Next
      ExcelObject\Invoke("Quit()") 
      WorkBook\Release()
    EndIf
  EndIf
  ExcelObject\Release()
Else
  MessageRequester("COMate -Excel demo", "Couldn't create the application object!")
EndIf


;Free our statement.
;===================
  COMate_FreeStatementHandle(hStatement)

; IDE Options = PureBasic 4.60 Beta 1 (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 17
; FirstLine = 10
; EnableUnicode
; Executable = nxReportU.dll