﻿;/////////////////////////////////////////////////////////////////////////////////
;***COMate***  COM automation through iDispatch.
;*===========
;*
;Script control (VBS). By srod.
;*
;*This little include allows us to create VB scripts (running in the MS Script Control) which can call PB functions.
;*See the demo 'Demo_Script_VBS - call PB function.pb' demo for an example of using this include.
;/////////////////////////////////////////////////////////////////////////////////

;/////////////////////////////////////////////////////////////////////////////////
;To use this include (as you will see in the aforementioned demo), create a scripting object and set the language to 'VBScript'.
;Then call the function in this include : 
;
;     CreateAddinObject(scriptObject.COMateObject, eventName$, eventProc)
;
;The first parameter is the scripting object mentioned previously (see demo).
;The second parameter is the name of a method you raise against the 'InvokePB' script object.
;E.g if you set eventName$ to "OurEvents" then to call our PB evepnt handler, our VB script needs to issue the following
;statement :    InvokePB.OurEvents param1, param2... etc.  (see demo).
;The third parameter is the address of our PB function which will receive the 'event calls'.
;
;Note that your script can pass as many parameters as you wish and this include attempts to convert them all to strings.
;/////////////////////////////////////////////////////////////////////////////////


#DISPID_PBEVENT = 1  ;Use any value here. This identifies our method of the 'InvokePB' object which we add To the scripting object.
                     ;Add additional values if you want to use additional methods etc.


Prototype VBSObject_EventCallback(numArgs, Array args$(1))


;Our iDispatch class template.
  Structure _VBSObject
    *vTable
    refCount.i
    eventName$
    eventProc.VBSObject_EventCallback
  EndStructure


;/////////////////////////////////////////////////////////////////////////////////
;Call this function and pass a COMate object housing a MSScriptControl object pointer and an event name for use
;in any scripts you might apply to the scriptObject.
Procedure.i CreateAddinObject(scriptObject.COMateObject, eventName$, eventProc)
  Protected *this._VBSObject
  If scriptObject And eventName$
    *this = AllocateMemory(SizeOf(_VBSObject))
    If *this
      With *this
        \vTable = ?VTable_VBSObjectiDispatch
        \eventProc = eventProc
      EndWith
      If scriptObject\Invoke("AddObject('InvokePB', " + Str(*this) + " as iDispatch, 1)") <> #S_OK
        FreeMemory(*this)
        *this = 0
      Else
        *this\eventName$ = eventName$
      EndIf
    EndIf
  EndIf
  ProcedureReturn *this
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////
  

;-VBSObject iDispatch implementation.

;/////////////////////////////////////////////////////////////////////////////////
Procedure.i VBSObject_QueryInterface(*this._VBSObject, iid, *ppvObject.INTEGER)
  Protected result
  If CompareMemory(iid, ?IID_IUnknown, SizeOf(CLSID)) Or CompareMemory(iid, ?IID_IDispatch, SizeOf(CLSID))
    *ppvObject\i = *this
    *this\refCount + 1
    result = #S_OK
  Else
    *ppvObject\i=0
    result = #E_NOINTERFACE
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i VBSObject_AddRef(*this._VBSObject)
  *this\refCount + 1
  ProcedureReturn *this\refCount
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i VBSObject_Release(*this._VBSObject)
  Protected result
  *this\refCount - 1
  If *this\refCount > 0
    result = *this\refCount
  Else
    ClearStructure(*this, _VBSObject)
    FreeMemory(*this)
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i VBSObject_GetTypeInfoCount(*this._VBSObject, *pctinfo.INTEGER) 
  *pctinfo\i = 0
  ProcedureReturn #S_OK 
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i VBSObject_GetTypeInfo(*this._VBSObject, itinfo, lcid, *ppTInfo.INTEGER) 
  ProcedureReturn #E_NOTIMPL 
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i VBSObject_GetIDsOfNames(*this._VBSObject, *riid, *rgszNames, cNames, lcid, *DispID.LONG) 
  Protected result = #S_OK , name$
  If cNames = 1
    name$ = LCase(PeekS(PeekI(*rgszNames), -1, #PB_Unicode))
    If name$ = LCase(*this\eventName$)
      *DispID\l = #DISPID_PBEVENT
    Else
      *DispID\l = -1 ;Unknown!
      result = #DISP_E_UNKNOWNNAME
    EndIf
  Else
    result = #E_FAIL  
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;Here is where we receive our object method calls.
;We pass all paramaters to our event callback in the form of a string array.
Procedure.i VBSObject_Invoke(*this._VBSObject, dispid, *riid, lcid, wflags.w, *Params.DISPPARAMS, *Result.VARIANT, *pExept, *ArgErr) 
  Protected result = #S_OK, dispGetResult, i, numArgs, Dim args$(0), var.VARIANT, puArgErr
  If dispid = #DISPID_PBEVENT
    If wflags =#DISPATCH_METHOD
      numArgs = *Params\cArgs + *Params\cNamedArgs
      Dim args$(numArgs)
      For i = 1 To numArgs
        result = DispGetParam_(*Params, i-1, #VT_BSTR, var, @puArgErr)      
        If result = #S_OK
          args$(i-1) = PeekS(var\bstrVal, -1, #PB_Unicode)        
          SysFreeString_(var\bstrVal)
        ElseIf result <> #DISP_E_TYPEMISMATCH
          result = #DISP_E_BADVARTYPE
          Break
        EndIf
      Next
      If result = #S_OK
        ;We can now call our event handler.
          *this\eventProc(numArgs, args$())
      EndIf
    Else
      result = #DISP_E_MEMBERNOTFOUND  
    EndIf
  Else
    result = #DISP_E_MEMBERNOTFOUND  
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


DataSection 
  VTable_VBSObjectiDispatch:
    Data.i @VBSObject_QueryInterface()
    Data.i @VBSObject_AddRef()
    Data.i @VBSObject_Release()
    Data.i @VBSObject_GetTypeInfoCount() 
    Data.i @VBSObject_GetTypeInfo() 
    Data.i @VBSObject_GetIDsOfNames() 
    Data.i @VBSObject_Invoke()
EndDataSection

; IDE Options = PureBasic 5.21 LTS (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 59
; FirstLine = 39
; Folding = --
; EnableUnicode
; Executable = nxReportU.dll